@@ -3,7 +3,7 @@
  * This class provides an object-oriented representation of a MIME part
  * (defined by RFC 2045).
  *
- * Copyright 1999-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2012 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -51,7 +51,9 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * @var array
      */
     static public $encodingTypes = array(
-        '7bit', '8bit', 'base64', 'binary', 'quoted-printable'
+        '7bit', '8bit', 'base64', 'binary', 'quoted-printable',
+        // Non-RFC types, but old mailers may still use
+        'uuencode', 'x-uuencode', 'x-uue'
     );
 
     /**
@@ -286,7 +288,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * Get the content-disposition of this part.
      *
      * @return string  The part's content-disposition.  An empty string means
-     * q               no desired disposition has been set for this part.
+     *                 no desired disposition has been set for this part.
      */
     public function getDisposition()
     {
@@ -374,13 +376,11 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * @param mixed $contents  The part body. Either a string or a stream
      *                         resource, or an array containing both.
      * @param array $options   Additional options:
-     * <pre>
-     * 'encoding' - (string) The encoding of $contents.
-     *              DEFAULT: Current transfer encoding value.
-     * 'usestream' - (boolean) If $contents is a stream, should we directly
-     *               use that stream?
-     *               DEFAULT: $contents copied to a new stream.
-     * </pre>
+     *   - encoding: (string) The encoding of $contents.
+     *               DEFAULT: Current transfer encoding value.
+     *   - usestream: (boolean) If $contents is a stream, should we directly
+     *                use that stream?
+     *                DEFAULT: $contents copied to a new stream.
      */
     public function setContents($contents, $options = array())
     {
@@ -402,13 +402,11 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      *
      * @param mixed $contents   The part body. Either a string or a stream
      *                          resource, or an array containing both.
-     * <pre>
-     * 'encoding' - (string) The encoding of $contents.
-     *              DEFAULT: Current transfer encoding value.
-     * 'usestream' - (boolean) If $contents is a stream, should we directly
-     *               use that stream?
-     *               DEFAULT: $contents copied to a new stream.
-     * </pre>
+     *   - encoding: (string) The encoding of $contents.
+     *               DEFAULT: Current transfer encoding value.
+     *   - usestream: (boolean) If $contents is a stream, should we directly
+     *                use that stream?
+     *                DEFAULT: $contents copied to a new stream.
      */
     public function appendContents($contents, $options = array())
     {
@@ -440,17 +438,15 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * Return the body of the part.
      *
      * @param array $options  Additional options:
-     * <pre>
-     * 'canonical' - (boolean) Returns the contents in strict RFC 822 &
-     *               2045 output - namely, all newlines end with the
-     *               canonical <CR><LF> sequence.
-     *               DEFAULT: No
-     * 'stream' - (boolean) Return the body as a stream resource.
-     *            DEFAULT: No
-     * </pre>
+     *   - canonical: (boolean) Returns the contents in strict RFC 822 &
+     *                2045 output - namely, all newlines end with the
+     *                canonical <CR><LF> sequence.
+     *                DEFAULT: No
+     *   - stream: (boolean) Return the body as a stream resource.
+     *             DEFAULT: No
      *
-     * @return mixed  The body text of the part, or a stream resource if
-     *                'stream' is true.
+     * @return mixed  The body text (string) of the part, null if there is no
+     *                contents, and a stream resource if 'stream' is true.
      */
     public function getContents($options = array())
     {
@@ -474,33 +470,39 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         if (ftell($fp)) {
             switch ($encoding) {
             case 'base64':
-                return $this->_writeStream($fp, array(
-                    'filter' => array(
-                        'convert.base64-decode' => array()
-                    )
-                ));
+                try {
+                    return $this->_writeStream($fp, array(
+                        'error' => true,
+                        'filter' => array(
+                            'convert.base64-decode' => array()
+                        )
+                    ));
+                } catch (ErrorException $e) {}
+
+                rewind($fp);
+                return $this->_writeStream(base64_decode(stream_get_contents($fp)));
 
             case 'quoted-printable':
                 try {
-                    $stream = $this->_writeStream($fp, array(
+                    return $this->_writeStream($fp, array(
                         'error' => true,
                         'filter' => array(
                             'convert.quoted-printable-decode' => array()
                         )
                     ));
-                } catch (ErrorException $e) {
-                    // Workaround for Horde Bug #8747
-                    rewind($fp);
-                    $stream = $this->_writeStream(quoted_printable_decode(stream_get_contents($fp)));
-                }
-                return $stream;
+                } catch (ErrorException $e) {}
+
+                // Workaround for Horde Bug #8747
+                rewind($fp);
+                return $this->_writeStream(quoted_printable_decode(stream_get_contents($fp)));
 
             case 'uuencode':
             case 'x-uuencode':
             case 'x-uue':
                 /* Support for uuencoded encoding - although not required by
                  * RFCs, some mailers may still encode this way. */
-                return $this->_writeStream(convert_uuencode($this->_readStream($fp)));
+                $res = Horde_Mime::uudecode($this->_readStream($fp));
+                return $this->_writeStream($res[0]['data']);
             }
         }
 
@@ -585,7 +587,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
       *
       * @param boolean $charset  Append character set information to the end
       *                          of the content type if this is a text/* part?
-      *
+      *`
       * @return string  The mimetype of this part (ex.: text/plain;
       *                 charset=us-ascii) or false.
       */
@@ -670,7 +672,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
     public function getCharset()
     {
         $charset = $this->getContentTypeParameter('charset');
-        if (is_null($charset)) {
+        if (is_null($charset) && $this->getPrimaryType() != 'text') {
             return null;
         }
 
@@ -678,7 +680,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
 
         if ($this->getPrimaryType() == 'text') {
             $d_charset = Horde_String::lower(self::$defaultCharset);
-            if (($d_charset != 'us-ascii') && ($charset == 'us-ascii')) {
+            if ($d_charset != 'us-ascii' &&
+                (!$charset || $charset == 'us-ascii')) {
                 return $d_charset;
             }
         }
@@ -801,10 +804,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      *
      * @param string $encoding  The transfer encoding to use.
      * @param array $options    Additional options:
-     * <pre>
-     * 'send' - (boolean) If true, use $encoding as the sending encoding.
-     *          DEFAULT: $encoding is used to change the base encoding.
-     * </pre>
+     *   - send: (boolean) If true, use $encoding as the sending encoding.
+     *           DEFAULT: $encoding is used to change the base encoding.
      */
     public function setTransferEncoding($encoding, $options = array())
     {
@@ -1013,12 +1014,11 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * for the part.
      *
      * @param array $options  Additional options:
-     * <pre>
-     * 'encode' - (integer) A mask of allowable encodings.
-     *            DEFAULT: See self::_getTransferEncoding()
-     * 'headers' - (Horde_Mime_Headers) The object to add the MIME headers to.
-     *             DEFAULT: Add headers to a new object
-     * </pre>
+     *   - encode: (integer) A mask of allowable encodings.
+     *             DEFAULT: See self::_getTransferEncoding()
+     *   - headers: (Horde_Mime_Headers) The object to add the MIME headers
+     *              to.
+     *              DEFAULT: Add headers to a new object
      *
      * @return Horde_Mime_Headers  A Horde_Mime_Headers object.
      */
@@ -1069,7 +1069,9 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         $disposition = $this->getDisposition();
         $disp_params = $this->getAllDispositionParameters();
         $name = $this->getName();
-        if ($disposition || !empty($name) || !empty($disp_params)) {
+        $disp_params_copy = $disp_params;
+        unset($disp_params_copy['size']);
+        if ($disposition || !empty($name) || !empty($disp_params_copy)) {
             if (!$disposition) {
                 $disposition = 'attachment';
             }
@@ -1102,26 +1104,24 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * Return the entire part in MIME format.
      *
      * @param array $options  Additional options:
-     * <pre>
-     * 'canonical' - (boolean) Returns the encoded part in strict RFC 822 &
-     *               2045 output - namely, all newlines end with the canonical
-     *               <CR><LF> sequence.
-     *               DEFAULT: false
-     * 'defserver' - (string) The default server to use when creating the
-     *               header string.
-     *               DEFAULT: none
-     * 'encode' - (integer) A mask of allowable encodings.
-     *            DEFAULT: self::ENCODE_7BIT
-     * 'headers' - (mixed) Include the MIME headers? If true, create a new
-     *             headers object. If a Horde_Mime_Headers object, add MIME
-     *             headers to this object. If a string, use the string
-     *             verbatim.
-     *             DEFAULT: true
-     * 'id' - (string) Return only this MIME ID part.
-     *        DEFAULT: Returns the base part.
-     * 'stream' - (boolean) Return a stream resource.
-     *            DEFAULT: false
-     * </pre>
+     *   - canonical: (boolean) Returns the encoded part in strict RFC 822 &
+     *                2045 output - namely, all newlines end with the
+     *                canonical <CR><LF> sequence.
+     *                DEFAULT: false
+     *   - defserver: (string) The default server to use when creating the
+     *                header string.
+     *                DEFAULT: none
+     *   - encode: (integer) A mask of allowable encodings.
+     *             DEFAULT: self::ENCODE_7BIT
+     *   - headers: (mixed) Include the MIME headers? If true, create a new
+     *              headers object. If a Horde_Mime_Headers object, add MIME
+     *              headers to this object. If a string, use the string
+     *              verbatim.
+     *              DEFAULT: true
+     *   - id: (string) Return only this MIME ID part.
+     *         DEFAULT: Returns the base part.
+     *   - stream: (boolean) Return a stream resource.
+     *             DEFAULT: false
      *
      * @return mixed  The MIME string (returned as a resource if $stream is
      *                true).
@@ -1180,7 +1180,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             $this->_temp['toString'] = '';
             $options['_baseptr'] = &$this->_temp['toString'];
 
-            /* Any information about a message/* is embedded in the message
+            /* Any information about a message is embedded in the message
              * contents themself. Simply output the contents of the part
              * directly and return. */
             $ptype = $this->getPrimaryType();
@@ -1247,6 +1247,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         }
 
         $newfp = $this->_writeStream($parts);
+
         array_map('fclose', $parts_close);
 
         if (!is_null($oldbaseptr)) {
@@ -1396,8 +1397,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * Determine the size of this MIME part and its child members.
      *
      * @param boolean $approx  If true, determines an approximate size for
-     *                         parts consisting of base64 encoded data (since
-     *                         1.1.0).
+     *                         parts consisting of base64 encoded data.
      *
      * @return integer  Size of the part, in bytes.
      */
@@ -1451,8 +1451,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * Output the size of this MIME part in KB.
      *
      * @param boolean $approx  If true, determines an approximate size for
-     *                         parts consisting of base64 encoded data (since
-     *                         1.1.0).
+     *                         parts consisting of base64 encoded data.
      *
      * @return string  Size of the part in KB.
      */
@@ -1495,7 +1494,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         if (is_null($this->_contentid)) {
             $this->_contentid = is_null($cid)
                 ? (strval(new Horde_Support_Randomid()) . '@' . $_SERVER['SERVER_NAME'])
-                : $cid;
+                : trim($cid, '<>');
         }
 
         return $this->_contentid;
@@ -1657,15 +1656,20 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
     /**
      * Sends this message.
      *
-     * @param string $email                The address list to send to.
-     * @param Horde_Mime_Headers $headers  The Horde_Mime_Headers object
-     *                                     holding this message's headers.
-     * @param Horde_Mail_Transport $mailer A Horde_Mail_Transport object.
+     * @param string $email                 The address list to send to.
+     * @param Horde_Mime_Headers $headers   The Horde_Mime_Headers object
+     *                                      holding this message's headers.
+     * @param Horde_Mail_Transport $mailer  A Horde_Mail_Transport object.
+     * @param array $opts                   Additional options:
+     *   - encode: (integer) The encoding to use. A mask of self::ENCODE_*
+     *             values.
+     *             DEFAULT: Auto-determined based on transport driver.
      *
      * @throws Horde_Mime_Exception
      * @throws InvalidArgumentException
      */
-    public function send($email, $headers, Horde_Mail_Transport $mailer)
+    public function send($email, $headers, Horde_Mail_Transport $mailer,
+                         array $opts = array())
     {
         $old_basepart = $this->_basepart;
         $this->_basepart = true;
@@ -1674,16 +1678,21 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
          * BINARYMIME (RFC 3030) extensions? Requires Net_SMTP version
          * 1.3+. */
         $encode = self::ENCODE_7BIT;
-        if ($mailer instanceof Horde_Mail_Transport_Smtp) {
-            try {
-                $smtp_ext = $mailer->getSMTPObject()->getServiceExtensions();
-                if (isset($smtp_ext['8BITMIME'])) {
-                    $encode |= self::ENCODE_8BIT;
-                }
-                if (isset($smtp_ext['BINARYMIME'])) {
-                    $encode |= self::ENCODE_BINARY;
-                }
-            } catch (Horde_Mail_Exception $e) {}
+        if (isset($opts['encode'])) {
+            /* Always allow 7bit encoding. */
+            $encode |= $opts['encode'];
+        } else {
+            if ($mailer instanceof Horde_Mail_Transport_Smtp) {
+                try {
+                    $smtp_ext = $mailer->getSMTPObject()->getServiceExtensions();
+                    if (isset($smtp_ext['8BITMIME'])) {
+                        $encode |= self::ENCODE_8BIT;
+                    }
+                    if (isset($smtp_ext['BINARYMIME'])) {
+                        $encode |= self::ENCODE_BINARY;
+                    }
+                } catch (Horde_Mail_Exception $e) {}
+            }
         }
 
         $msg = $this->toString(array(
@@ -1727,9 +1736,12 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         }
 
         $this->_basepart = $old_basepart;
-
+        $rfc822 = new Horde_Mail_Rfc822();
         try {
-            $mailer->send(Horde_Mime::encodeAddress($email, $this->getCharset()), $headers->toArray(array(
+            $mailer->send($rfc822->parseAddressList($email)->writeAddress(array(
+                'encode' => $this->getHeaderCharset(),
+                'idn' => true
+            )), $headers->toArray(array(
                 'canonical' => true,
                 'charset' => $this->getHeaderCharset()
             )), $msg);
@@ -1745,7 +1757,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * @param string $subtype  Specifically search for this subtype.
      *
      * @return mixed  The MIME ID of the main body part, or null if a body
-                      part is not found.
+     *                part is not found.
      */
     public function findBody($subtype = null)
     {
@@ -1771,14 +1783,12 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      * @param array $data     The data to write. Either a stream resource or
      *                        a string.
      * @param array $options  Additional options:
-     * <pre>
-     * error - (boolean) Catch errors when writing to the stream. Throw an
-     *         ErrorException if an error is found.
-     *         DEFAULT: false
-     * filter - (array) Filter(s) to apply to the string. Keys are the
-     *          filter names, values are filter params.
-     * fp - (resource) Use this stream instead of creating a new one.
-     * </pre>
+     *   - error: (boolean) Catch errors when writing to the stream. Throw an
+     *            ErrorException if an error is found.
+     *            DEFAULT: false
+     *   - filter: (array) Filter(s) to apply to the string. Keys are the
+     *             filter names, values are filter params.
+     *   - fp: (resource) Use this stream instead of creating a new one.
      *
      * @return resource  The stream resource.
      * @throws ErrorException
@@ -1931,12 +1941,10 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      *
      * @param string $text    The text of the MIME message.
      * @param array $options  Additional options:
-     * <pre>
-     * 'forcemime' - (boolean) If true, the message data is assumed to be
-     *               MIME data. If not, a MIME-Version header must exist (RFC
-     *               2045 [4]) to be parsed as a MIME message.
-     *               DEFAULT: false
-     * </pre>
+     *   - forcemime: (boolean) If true, the message data is assumed to be
+     *                MIME data. If not, a MIME-Version header must exist (RFC
+     *                2045 [4]) to be parsed as a MIME message.
+     *                DEFAULT: false
      *
      * @return Horde_Mime_Part  A MIME Part object.
      * @throws Horde_Mime_Exception
@@ -1978,7 +1986,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
 
             if (!empty($body)) {
                 $ob->setContents($body);
-                $ob->setBytes( strlen(str_replace(array("\r\n", "\n"), array("\n", "\r\n"), $body)));
+                $ob->setBytes(strlen(str_replace(array("\r\n", "\n"), array("\n", "\r\n"), $body)));
             }
 
             return $ob;
@@ -2026,7 +2034,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         }
 
         /* Get file size (if 'body' text is set). */
-        if (!empty($body)) {
+        if (!empty($body) && $ob->getPrimaryType() != 'multipart') {
             $ob->setContents($body);
             if ($ob->getType() != '/message/rfc822') {
                 $ob->setBytes(strlen(str_replace(array("\r\n", "\n"), array("\n", "\r\n"), $body)));
